import { Component, Input, Directive, ViewContainerRef, Renderer2, EventEmitter, ChangeDetectionStrategy, ComponentFactoryResolver, ChangeDetectorRef, Injector, ElementRef, Output, ViewChild, NgModule } from '@angular/core';
import { MatCheckbox, MatCheckboxModule } from '@angular/material/checkbox';
import { TdHighlightComponent, CovalentHighlightModule } from '@covalent/highlight';
import { scrollToAnchor, TdMarkdownComponent, MarkdownLoaderService, CovalentMarkdownModule } from '@covalent/markdown';
import { TdDataTableComponent, TdDataTableSortingOrder, CovalentDataTableModule } from '@covalent/core/data-table';
import { CommonModule } from '@angular/common';
import { MatListModule } from '@angular/material/list';
import { __awaiter } from 'tslib';
import { MatProgressBarModule } from '@angular/material/progress-bar';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TdFlavoredListComponent {
    constructor() {
        this.dense = false;
    }
}
TdFlavoredListComponent.decorators = [
    { type: Component, args: [{
                selector: 'cfm-list',
                template: "<mat-list [attr.dense]=\"dense ? true : null\">\n  <ng-template let-line let-last=\"last\" ngFor [ngForOf]=\"lines\">\n    <mat-list-item>\n      <h4 matLine>\n        {{ line.line }}\n      </h4>\n      <p *ngFor=\"let subline of line.sublines\" matLine>\n        {{ subline }}\n      </p>\n      <mat-divider *ngIf=\"!last\"></mat-divider>\n    </mat-list-item>\n  </ng-template>\n</mat-list>\n",
                styles: [""]
            }] }
];
TdFlavoredListComponent.propDecorators = {
    lines: [{ type: Input }],
    dense: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TdFlavoredMarkdownContainerDirective {
    /**
     * @param {?} viewContainerRef
     * @param {?} _renderer
     */
    constructor(viewContainerRef, _renderer) {
        this.viewContainerRef = viewContainerRef;
        this._renderer = _renderer;
    }
    /**
     * @return {?}
     */
    clear() {
        this._renderer.setProperty(this.viewContainerRef.element.nativeElement, 'innerHTML', '');
        this.viewContainerRef.clear();
    }
}
TdFlavoredMarkdownContainerDirective.decorators = [
    { type: Directive, args: [{
                selector: '[tdFlavoredMarkdownContainer]',
            },] }
];
/** @nocollapse */
TdFlavoredMarkdownContainerDirective.ctorParameters = () => [
    { type: ViewContainerRef },
    { type: Renderer2 }
];
class TdFlavoredMarkdownComponent {
    /**
     * @param {?} _componentFactoryResolver
     * @param {?} _renderer
     * @param {?} _changeDetectorRef
     * @param {?} _injector
     * @param {?} _elementRef
     */
    constructor(_componentFactoryResolver, _renderer, _changeDetectorRef, _injector, _elementRef) {
        this._componentFactoryResolver = _componentFactoryResolver;
        this._renderer = _renderer;
        this._changeDetectorRef = _changeDetectorRef;
        this._injector = _injector;
        this._elementRef = _elementRef;
        this._simpleLineBreaks = false;
        this._components = {};
        this._viewInit = false;
        /**
         * contentReady?: function
         * Event emitted after the markdown content rendering is finished.
         */
        this.onContentReady = new EventEmitter();
    }
    /**
     * content?: string
     *
     * Markdown format content to be parsed as material/covalent markup.
     *
     * e.g. README.md content.
     * @param {?} content
     * @return {?}
     */
    set content(content) {
        this._content = content;
    }
    /**
     * simpleLineBreaks?: string
     *
     * Sets whether newline characters inside paragraphs and spans are parsed as <br/>.
     * Defaults to false.
     * @param {?} simpleLineBreaks
     * @return {?}
     */
    set simpleLineBreaks(simpleLineBreaks) {
        this._simpleLineBreaks = simpleLineBreaks;
    }
    /**
     * hostedUrl?: string
     *
     * If markdown contains relative paths, this is required to generate correct urls
     *
     * @param {?} hostedUrl
     * @return {?}
     */
    set hostedUrl(hostedUrl) {
        this._hostedUrl = hostedUrl;
    }
    /**
     * anchor?: string
     *
     * Anchor to jump to
     *
     * @param {?} anchor
     * @return {?}
     */
    set anchor(anchor) {
        this._anchor = anchor;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        // only anchor changed
        if (changes.anchor && !changes.content && !changes.simpleLineBreaks && !changes.hostedUrl) {
            scrollToAnchor(this._elementRef.nativeElement, this._anchor, false);
        }
        else {
            this.refresh();
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (!this._content) {
            this._loadContent(((/** @type {?} */ (this.container.viewContainerRef.element.nativeElement))).textContent);
            Promise.resolve().then((/**
             * @return {?}
             */
            () => {
                this._viewInit = true;
                this._changeDetectorRef.markForCheck();
            }));
        }
    }
    /**
     * @return {?}
     */
    refresh() {
        if (this._content) {
            this._loadContent(this._content);
        }
        else if (this._viewInit) {
            this._loadContent(((/** @type {?} */ (this.container.viewContainerRef.element.nativeElement))).textContent);
        }
        this._changeDetectorRef.markForCheck();
    }
    /**
     * @private
     * @param {?} markdown
     * @return {?}
     */
    _loadContent(markdown) {
        if (markdown && markdown.trim().length > 0) {
            this.container.clear();
            markdown = markdown.replace(/^(\s|\t)*\n+/g, '').replace(/(\s|\t)*\n+(\s|\t)*$/g, '');
            // Split markdown by line characters
            /** @type {?} */
            let lines = markdown.split('\n');
            // check how much indentation is used by the first actual markdown line
            /** @type {?} */
            let firstLineWhitespace = lines[0].match(/^(\s|\t)*/)[0];
            // Remove all indentation spaces so markdown can be parsed correctly
            /** @type {?} */
            let startingWhitespaceRegex = new RegExp('^' + firstLineWhitespace);
            lines = lines.map((/**
             * @param {?} line
             * @return {?}
             */
            function (line) {
                return line.replace(startingWhitespaceRegex, '');
            }));
            // Join lines again with line characters
            markdown = [...lines, '', ''].join('\n');
            markdown = this._replaceCodeBlocks(markdown);
            markdown = this._replaceCheckbox(markdown);
            markdown = this._replaceTables(markdown);
            markdown = this._replaceLists(markdown);
            /** @type {?} */
            let keys = Object.keys(this._components);
            // need to sort the placeholders in order of encounter in markdown content
            keys = keys.sort((/**
             * @param {?} compA
             * @param {?} compB
             * @return {?}
             */
            (compA, compB) => {
                return markdown.indexOf(compA) > markdown.indexOf(compB) ? 1 : -1;
            }));
            this._render(markdown, keys[0], keys);
            // TODO: timeout required since resizing of html elements occurs which causes a change in the scroll position
            setTimeout((/**
             * @return {?}
             */
            () => scrollToAnchor(this._elementRef.nativeElement, this._anchor, false)), 250);
            this.onContentReady.emit();
            Promise.resolve().then((/**
             * @return {?}
             */
            () => {
                this._changeDetectorRef.markForCheck();
            }));
        }
    }
    /**
     * @private
     * @param {?} markdown
     * @param {?} key
     * @param {?} keys
     * @return {?}
     */
    _render(markdown, key, keys) {
        if (!markdown) {
            return;
        }
        if (key && markdown.indexOf(key) > -1) {
            /** @type {?} */
            let markdownParts = markdown.split(key);
            keys.shift();
            this._render(markdownParts[0], undefined, undefined);
            this.container.viewContainerRef.insert(this._components[key].hostView, this.container.viewContainerRef.length);
            this._components[key] = undefined;
            delete this._components[key];
            this._render(markdownParts[1], keys[0], keys);
        }
        else {
            /** @type {?} */
            let contentRef = this._componentFactoryResolver
                .resolveComponentFactory(TdMarkdownComponent)
                .create(this._injector);
            contentRef.instance.content = markdown;
            contentRef.instance.hostedUrl = this._hostedUrl;
            contentRef.instance.simpleLineBreaks = this._simpleLineBreaks;
            contentRef.instance.refresh();
            this.container.viewContainerRef.insert(contentRef.hostView, this.container.viewContainerRef.length);
        }
    }
    /**
     * @private
     * @template T
     * @param {?} markdown
     * @param {?} type
     * @param {?} regExp
     * @param {?} replacerFunc
     * @return {?}
     */
    _replaceComponent(markdown, type, regExp, replacerFunc) {
        /** @type {?} */
        let componentIndex = 0;
        return markdown.replace(regExp, (/**
         * @param {...?} args
         * @return {?}
         */
        (...args) => {
            /** @type {?} */
            let componentFactory = this._componentFactoryResolver.resolveComponentFactory(type);
            /** @type {?} */
            let componentRef = componentFactory.create(this._injector);
            replacerFunc(componentRef, ...args);
            /** @type {?} */
            let key = '[' + componentFactory.selector + '-placeholder-' + componentIndex++ + ']';
            this._components[key] = componentRef;
            return key;
        }));
    }
    /**
     * @private
     * @param {?} markdown
     * @return {?}
     */
    _replaceCheckbox(markdown) {
        /** @type {?} */
        let checkboxRegExp = /(?:^|\n)- \[(x| )\](.*)/gi;
        return this._replaceComponent(markdown, MatCheckbox, checkboxRegExp, (/**
         * @param {?} componentRef
         * @param {?} match
         * @param {?} checked
         * @param {?} label
         * @return {?}
         */
        (componentRef, match, checked, label) => {
            componentRef.instance.checked = !!checked.trim();
            componentRef.instance.disabled = true;
            componentRef.instance.labelPosition = 'after';
            this._renderer.setProperty(((/** @type {?} */ (componentRef.instance._elementRef.nativeElement))).getElementsByClassName('mat-checkbox-label')[0], 'innerHTML', label);
        }));
    }
    /**
     * @private
     * @param {?} markdown
     * @return {?}
     */
    _replaceCodeBlocks(markdown) {
        /** @type {?} */
        let codeBlockRegExp = /(?:^|\n)```(.*)\n([\s\S]*?)\n```/g;
        return this._replaceComponent(markdown, TdHighlightComponent, codeBlockRegExp, (/**
         * @param {?} componentRef
         * @param {?} match
         * @param {?} language
         * @param {?} codeblock
         * @return {?}
         */
        (componentRef, match, language, codeblock) => {
            if (language) {
                componentRef.instance.language = language;
            }
            componentRef.instance.content = codeblock;
        }));
    }
    /**
     * @private
     * @param {?} markdown
     * @return {?}
     */
    _replaceTables(markdown) {
        /** @type {?} */
        let tableRgx = /^ {0,3}\|?.+\|.+\n[ \t]{0,3}\|?[ \t]*:?[ \t]*(?:-|=){2,}[ \t]*:?[ \t]*\|[ \t]*:?[ \t]*(?:-|=){2,}[\s\S]+?(?:\n\n|~0)/gm;
        return this._replaceComponent(markdown, TdDataTableComponent, tableRgx, (/**
         * @param {?} componentRef
         * @param {?} match
         * @return {?}
         */
        (componentRef, match) => {
            /** @type {?} */
            let dataTableLines = match.replace(/(\s|\t)*\n+(\s|\t)*$/g, '').split('\n');
            /** @type {?} */
            let columns = dataTableLines[0]
                .split('|')
                .filter((/**
             * @param {?} col
             * @return {?}
             */
            (col) => {
                return col;
            }))
                .map((/**
             * @param {?} s
             * @return {?}
             */
            (s) => {
                return s.trim();
            }));
            /** @type {?} */
            let alignment = dataTableLines[1]
                .split('|')
                .filter((/**
             * @param {?} v
             * @return {?}
             */
            (v) => {
                return v;
            }))
                .map((/**
             * @param {?} s
             * @return {?}
             */
            (s) => {
                return s.trim();
            }));
            componentRef.instance.columns = columns.map((/**
             * @param {?} col
             * @param {?} index
             * @return {?}
             */
            (col, index) => {
                /** @type {?} */
                let widths = alignment[index].split('---');
                /** @type {?} */
                let min = parseInt(widths[0], 10);
                /** @type {?} */
                let max = parseInt(widths[1], 10);
                /** @type {?} */
                let width = { min: min, max: max };
                if (isNaN(min) && isNaN(max)) {
                    width = undefined;
                }
                else if (isNaN(max)) {
                    width.max = undefined;
                }
                else if (isNaN(min)) {
                    width.min = undefined;
                }
                return {
                    label: col,
                    name: col.toLowerCase().trim(),
                    numeric: /^--*[ \t]*:[ \t]*$/.test(alignment[index]),
                    width: width,
                };
            }));
            /** @type {?} */
            let data = [];
            for (let i = 2; i < dataTableLines.length; i++) {
                /** @type {?} */
                let rowSplit = dataTableLines[i]
                    .split('|')
                    .filter((/**
                 * @param {?} cell
                 * @return {?}
                 */
                (cell) => {
                    return cell;
                }))
                    .map((/**
                 * @param {?} s
                 * @return {?}
                 */
                (s) => {
                    return s.trim();
                }));
                /** @type {?} */
                let row = {};
                columns.forEach((/**
                 * @param {?} col
                 * @param {?} index
                 * @return {?}
                 */
                (col, index) => {
                    /** @type {?} */
                    const rowSplitCell = rowSplit[index];
                    if (rowSplitCell) {
                        row[col.toLowerCase().trim()] = rowSplitCell.replace(/`(.*)`/, (/**
                         * @param {?} m
                         * @param {?} value
                         * @return {?}
                         */
                        (m, value) => {
                            return value;
                        }));
                    }
                }));
                data.push(row);
            }
            componentRef.instance.data = data;
            componentRef.instance.sortable = true;
            componentRef.instance.onSortChange.subscribe((/**
             * @param {?} event
             * @return {?}
             */
            (event) => {
                componentRef.instance.data = data.sort((/**
                 * @param {?} a
                 * @param {?} b
                 * @return {?}
                 */
                (a, b) => {
                    /** @type {?} */
                    let compA = a[event.name];
                    /** @type {?} */
                    let compB = b[event.name];
                    /** @type {?} */
                    let direction = 0;
                    if (!Number.isNaN(Number.parseFloat(compA)) && !Number.isNaN(Number.parseFloat(compB))) {
                        direction = Number.parseFloat(compA) - Number.parseFloat(compB);
                    }
                    else {
                        if (compA < compB) {
                            direction = -1;
                        }
                        else if (compA > compB) {
                            direction = 1;
                        }
                    }
                    return direction * (event.order === TdDataTableSortingOrder.Descending ? -1 : 1);
                }));
                componentRef.instance.refresh();
            }));
            setTimeout((/**
             * @return {?}
             */
            () => {
                componentRef.instance.refresh();
            }));
        }));
    }
    /**
     * @private
     * @param {?} markdown
     * @return {?}
     */
    _replaceLists(markdown) {
        /** @type {?} */
        let listRegExp = /(?:^|\n)(( *\+)[ |\t](.*)\n)+/g;
        return this._replaceComponent(markdown, TdFlavoredListComponent, listRegExp, (/**
         * @param {?} componentRef
         * @param {?} match
         * @return {?}
         */
        (componentRef, match) => {
            /** @type {?} */
            let lineTexts = match.split(new RegExp('\\n {' + (match.indexOf('+') - 1).toString() + '}\\+[ |\\t]'));
            lineTexts.shift();
            /** @type {?} */
            let lines = [];
            lineTexts.forEach((/**
             * @param {?} text
             * @param {?} index
             * @return {?}
             */
            (text, index) => {
                /** @type {?} */
                let sublineTexts = text.split(/\n *\+ /);
                lines.push({
                    line: sublineTexts.shift(),
                    sublines: sublineTexts.map((/**
                     * @param {?} subline
                     * @return {?}
                     */
                    (subline) => {
                        return subline.trim();
                    })),
                });
            }));
            componentRef.instance.lines = lines;
        }));
    }
}
TdFlavoredMarkdownComponent.decorators = [
    { type: Component, args: [{
                selector: 'td-flavored-markdown',
                template: "<div tdFlavoredMarkdownContainer>\n  <ng-content></ng-content>\n</div>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [":host ::ng-deep td-markdown:first-of-type>div:first-of-type>h1:first-of-type,:host ::ng-deep td-markdown:first-of-type>div:first-of-type>h2:first-of-type{margin-top:0}:host ::ng-deep mat-checkbox,:host ::ng-deep td-data-table,:host ::ng-deep td-highlight{display:block}:host ::ng-deep mat-checkbox:last-of-type,:host ::ng-deep td-data-table,:host ::ng-deep td-highlight{margin-bottom:16px}:host ::ng-deep :not(pre)>code{font-family:Menlo,Monaco,\"Andale Mono\",\"lucida console\",\"Courier New\",monospace;padding-left:2px;padding-right:2px}"]
            }] }
];
/** @nocollapse */
TdFlavoredMarkdownComponent.ctorParameters = () => [
    { type: ComponentFactoryResolver },
    { type: Renderer2 },
    { type: ChangeDetectorRef },
    { type: Injector },
    { type: ElementRef }
];
TdFlavoredMarkdownComponent.propDecorators = {
    content: [{ type: Input, args: ['content',] }],
    simpleLineBreaks: [{ type: Input, args: ['simpleLineBreaks',] }],
    hostedUrl: [{ type: Input, args: ['hostedUrl',] }],
    anchor: [{ type: Input, args: ['anchor',] }],
    onContentReady: [{ type: Output, args: ['contentReady',] }],
    container: [{ type: ViewChild, args: [TdFlavoredMarkdownContainerDirective, { static: true },] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO: make a td-markdown-loader component
class TdFlavoredMarkdownLoaderComponent {
    /**
     * @param {?} _changeDetectorRef
     * @param {?} _markdownUrlLoaderService
     */
    constructor(_changeDetectorRef, _markdownUrlLoaderService) {
        this._changeDetectorRef = _changeDetectorRef;
        this._markdownUrlLoaderService = _markdownUrlLoaderService;
        /**
         * contentReady: void
         * Emitted when markdown rendering is finished.
         */
        this.contentReady = new EventEmitter();
        /**
         * loadFailed: Error
         * Emitted when loading of markdown file fails.
         */
        this.loadFailed = new EventEmitter();
        this.loading = true;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.url || changes.httpOptions) {
            this.loadMarkdown();
        }
    }
    /**
     * @return {?}
     */
    loadMarkdown() {
        return __awaiter(this, void 0, void 0, function* () {
            this.loading = true;
            try {
                this.content = yield this._markdownUrlLoaderService.load(this.url, this.httpOptions);
            }
            catch (error) {
                this.loadFailed.emit(error);
            }
            finally {
                this.loading = false;
                this._changeDetectorRef.markForCheck();
            }
        });
    }
}
TdFlavoredMarkdownLoaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'td-flavored-markdown-loader',
                template: "<mat-progress-bar *ngIf=\"loading\" mode=\"indeterminate\" color=\"accent\"></mat-progress-bar>\n<td-flavored-markdown\n  *ngIf=\"!loading\"\n  [content]=\"content\"\n  [hostedUrl]=\"url\"\n  [anchor]=\"anchor\"\n  (contentReady)=\"contentReady.emit()\"\n></td-flavored-markdown>\n",
                styles: [":host{display:block}"]
            }] }
];
/** @nocollapse */
TdFlavoredMarkdownLoaderComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: MarkdownLoaderService }
];
TdFlavoredMarkdownLoaderComponent.propDecorators = {
    url: [{ type: Input }],
    httpOptions: [{ type: Input }],
    anchor: [{ type: Input }],
    contentReady: [{ type: Output }],
    loadFailed: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CovalentFlavoredMarkdownModule {
}
CovalentFlavoredMarkdownModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MatCheckboxModule,
                    MatListModule,
                    MatProgressBarModule,
                    CovalentDataTableModule,
                    CovalentHighlightModule,
                    CovalentMarkdownModule,
                ],
                declarations: [
                    TdFlavoredListComponent,
                    TdFlavoredMarkdownComponent,
                    TdFlavoredMarkdownContainerDirective,
                    TdFlavoredMarkdownLoaderComponent,
                ],
                exports: [TdFlavoredMarkdownComponent, TdFlavoredMarkdownLoaderComponent],
                entryComponents: [
                    TdDataTableComponent,
                    TdMarkdownComponent,
                    TdHighlightComponent,
                    MatCheckbox,
                    TdFlavoredListComponent,
                ],
            },] }
];

export { CovalentFlavoredMarkdownModule, TdFlavoredMarkdownComponent, TdFlavoredMarkdownContainerDirective, TdFlavoredMarkdownLoaderComponent, TdFlavoredListComponent as ɵa };
//# sourceMappingURL=covalent-flavored-markdown.js.map
